from ngsolve import *
from netgen.geom2d import SplineGeometry
from math import pi

geo = SplineGeometry()
geo.AddCircle( (0, 0), r=0.5, bc = "cyl")
mesh = Mesh(geo.GenerateMesh(maxh=0.1))
mesh.Curve(3)
Draw(mesh)

fes = VectorH1(mesh, order=3)

u_ex = GridFunction(fes)
u_ex.Set(CoefficientFunction( (x**2+y**2, 0) ))

VH1 = H1(mesh, order=2)

gradu00 = GridFunction(VH1)
gradu01 = GridFunction(VH1)
gradu10 = GridFunction(VH1)
gradu11 = GridFunction(VH1)

gradu00.Set(grad(u_ex.components[0])[0])
gradu01.Set(grad(u_ex.components[0])[1])
gradu10.Set(grad(u_ex.components[1])[0])
gradu11.Set(grad(u_ex.components[1])[1])

n = specialcf.normal(mesh.dim)
flux_x = Integrate( gradu00*n[0] + gradu01*n[1], mesh, BND)
flux_y = Integrate( gradu10*n[0] + gradu11*n[1], mesh, BND)

print ("flux_x:", flux_x)
print ("flux_y:", flux_y)

print("err = ", pi-flux_x)
